﻿<PropertyTab(GetType(Design.EventBindingsPropertyTab), PropertyTabScope.Document)>
Public Class SampleDataSource
    Implements INotifyPropertyChanged

#Region "Fields & Events"
    Public Event PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs) Implements INotifyPropertyChanged.PropertyChanged

    Private _LastClickedSender As Object
    Private _LastMouseHoverSender As Object
    Private _LastMouseMoveSender As Object
#End Region

#Region "Properties"
    Public Property Owner As SampleForm
    Public Property Binder As IEventBinder
    Public Property TraceTextBox As TextBox
    Public Property LongMethodIteration As Int32 = 50000
    Public Property StopLongMethod As Boolean
    Public ReadOnly Property CanBindHandlers As Boolean
        Get
            Return Not Me._Binder.Handled
        End Get
    End Property
    Public ReadOnly Property CanUnbindHandlers As Boolean
        Get
            Return Me._Binder.Handled
        End Get
    End Property
    Public ReadOnly Property LastClickedSender As String
        Get
            Return Me.TryGetName(Me._LastClickedSender)
        End Get
    End Property
    Public ReadOnly Property LastMouseHoverSender As String
        Get
            Return Me.TryGetName(Me._LastMouseHoverSender)
        End Get
    End Property
    Public ReadOnly Property LastMouseMoveSender As String
        Get
            Return Me.TryGetName(Me._LastMouseMoveSender)
        End Get
    End Property
#End Region

#Region "Methods"
    Private Function TryGetName(ByVal sender As Object) As String
        If TypeOf sender Is Control Then
            Return DirectCast(sender, Control).Name
        End If
        Return String.Empty
    End Function
    Private Sub NotifyPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

    Public Sub ClearTraces()
        Me._TraceTextBox.Clear()
    End Sub
    Public Sub BindHandlers()
        Me._Binder.AddHandlers()
        Me.NotifyPropertyChanged("CanBindHandlers")
        Me.NotifyPropertyChanged("CanUnbindHandlers")
    End Sub
    Public Sub UnbindHandlers()
        Me._Binder.RemoveHandlers()
        Me.NotifyPropertyChanged("CanBindHandlers")
        Me.NotifyPropertyChanged("CanUnbindHandlers")
    End Sub

    Private Sub AddTrace(ByVal eventName As String, ByVal senderName As String, ByVal args As String)
        If Not String.IsNullOrEmpty(senderName) Then
            Me.AddTrace(String.Format("{0} on {1}: {2}", eventName, senderName, args))
        Else
            Me.AddTrace(String.Format("{0}: {2}", eventName, args))
        End If
    End Sub
    Public Sub AddTrace(ByVal value As String)
        Me._TraceTextBox.AppendText(value + ControlChars.CrLf)
    End Sub
    Public Sub TraceClick(ByVal sender As Object)
        Me.AddTrace("Click", Me.TryGetName(sender), "")
        Me._LastClickedSender = sender
        Me.NotifyPropertyChanged("LastClickedSender")
    End Sub
    Public Sub TraceMouseHover(ByVal sender As Object, ByVal e As EventArgs)
        Me.AddTrace("Mouse Hover", Me.TryGetName(sender), e.ToString)
        Me._LastMouseHoverSender = sender
        Me.NotifyPropertyChanged("LastMouseHoverSender")
    End Sub
    Public Sub TraceMouseMove(ByVal e As MouseEventArgs, ByVal sender As Object)
        Me.AddTrace("Mouse Move", Me.TryGetName(sender), e.Location.ToString)
        Me._LastMouseMoveSender = sender
        Me.NotifyPropertyChanged("LastMouseMoveSender")
    End Sub

    Public Sub InterromptLongMethod()
        Me.StopLongMethod = True
    End Sub
    Public Sub LongMethod()
        Me.StopLongMethod = False
        If Me.LongMethodIteration < 0 Then Me.LongMethodIteration = -Me.LongMethodIteration
        For i = 0 To Me.LongMethodIteration
            Me._Owner.SetText(String.Format("Commande longue, étape: {0}/{1}", i, Me.LongMethodIteration))
            If Me.StopLongMethod Then
                Me._Owner.SetText("Stopped")
                Exit For
            End If
        Next
        Me.StopLongMethod = True
    End Sub
#End Region

End Class
