﻿Imports EventBindings.Extensions
Imports System.Reflection

Public Class MethodMediatorFactory
    Implements IEventMediatorFactory

    Public Synchronizer As ISynchronizeInvoke

    Public Function SupportMediationFor(ByVal [event] As EventData, ByVal handling As HandlingData, ByVal mediationMode As MediationMode) As Boolean Implements IEventMediatorFactory.SupportMediationFor
        If mediationMode = EventBindings.MediationMode.Synchronized AndAlso Me.Synchronizer Is Nothing Then Return False
        If Not Me.IsSupportedEvent([event].EventDescriptor) Then Return False
        If handling Is Nothing Then Return False
        Return handling.EventHandlerType.IsType(Of MethodInfo)()
    End Function
    Private Function IsSupportedEvent(ByVal descriptor As EventDescriptor) As Boolean
        If descriptor Is Nothing Then Return False
        If descriptor.IsEventHandler Then Return True
        If descriptor.IsGenericEventHandler Then Return True
        If descriptor.IsCustomEventHandler Then Return True
        Return False
    End Function

    Public Function CreateMediator(ByVal [event] As EventData, ByVal handling As HandlingData, ByVal mediationMode As MediationMode) As IEventMediator Implements IEventMediatorFactory.CreateMediator
        Dim Mediator = Me.CreateMediator([event])
        If Mediator Is Nothing Then Throw New NotSupportedException
        Mediator.Initialize([event])
        Mediator.Initialize(DirectCast(handling.EventHandler, MethodInfo), handling.EventHandlerSource)
        Mediator.Initialize(mediationMode, Me.Synchronizer)
        Return Mediator
    End Function
    Private Function CreateMediator(ByVal [event] As EventData) As IMethodMediator
        If [event].EventDescriptor.IsEventHandler Then Return Me.CreateStandardMediator
        If [event].EventDescriptor.IsGenericEventHandler Then Return Me.CreateGenericMediator([event])
        Return Me.TryCreateCustomMediator([event])
    End Function
    Private Function CreateStandardMediator() As IMethodMediator
        Return New MethodMediator
    End Function
    Private Function CreateGenericMediator(ByVal [event] As EventData) As IMethodMediator
        Dim MediatorArg = [event].EventDescriptor.EventType.GetGenericArguments(0)
        Dim MediatorGenDef = GetType(MethodMediator(Of ))
        Dim MediatorType = MediatorGenDef.MakeGenericType(New Type() {MediatorArg})
        Dim Mediator = DirectCast(Activator.CreateInstance(MediatorType), IMethodMediator)
        Return Mediator
    End Function
    Private Function TryCreateCustomMediator(ByVal [event] As EventData) As IMethodMediator
        Dim MediatorArg = [event].EventDescriptor.GetCustomEventArgsType
        If MediatorArg Is Nothing Then Return Nothing
        Dim MediatorGenDef = GetType(CustomMethodMediator(Of ))
        Dim MediatorType = MediatorGenDef.MakeGenericType(New Type() {MediatorArg})
        Dim Mediator = DirectCast(Activator.CreateInstance(MediatorType), IMethodMediator)
        Return Mediator
    End Function

End Class
