﻿Imports System.ComponentModel.Design
Imports EventBindings.Extensions
Namespace Design
    Public Class EventBindingsBox

#Region "Fields"
        Private _BindingsController As ListBoxController(Of EventBinding)
        Private _Bindings As List(Of EventBinding)
        Private _Provider As IEventBindingServiceProvider
        Private _ChangeService As IComponentChangeService
#End Region

#Region "Constructor"
        Public Sub New()

            ' Cet appel est requis par le concepteur.
            InitializeComponent()

            ' Ajoutez une initialisation quelconque après l'appel InitializeComponent().
            Me._BindingsController = New ListBoxController(Of EventBinding)(Me.BindingsListBox, AddressOf OnBindingChanged)
            Me._Bindings = New List(Of EventBinding)
        End Sub
#End Region

#Region "Properties"
        <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)>
        Public Property EventDataForCreation As EventData
        Public ReadOnly Property Bindings As EventBinding()
            Get
                Return Me._Bindings.ToArray
            End Get
        End Property
#End Region

#Region "Methods"
        Private Sub NotifyComponentsChanged(ByVal components As IEnumerable)
            If components.IsNullOrEmpty OrElse Me._ChangeService Is Nothing Then Exit Sub
            For Each Item In components
                Me.NotifyComponentChanged(Item)
            Next
        End Sub
        Private Sub NotifyComponentChanged(ByVal component As Object)
            If Me._ChangeService Is Nothing OrElse component Is Nothing Then Exit Sub
            Me._ChangeService.OnComponentChanged(component, Nothing, Nothing, Nothing)
        End Sub
        Public Sub Fill(ByVal bindings As IEnumerable(Of EventBinding), ByVal provider As IEventBindingServiceProvider)
            Me._Provider = provider
            Me._BindingsController.ReferenceService = provider.GetService(Of IReferenceService)()
            Me._ChangeService = provider.GetService(Of IComponentChangeService)()
            If bindings.IsNullOrEmpty Then
                Me._Bindings.Clear()
                Me.RefreshBindings(Nothing)
            Else
                Me._Bindings.AddRange(bindings)
                Me.RefreshBindings(Me._Bindings.First)
            End If
            Me.NewToolStripButton.Visible = Me.EventDataForCreation.IsValidEvent
        End Sub
        Private Sub RefreshBindings(ByVal selection As EventBinding)
            Me._BindingsController.Items = Me._Bindings
            Me._BindingsController.SelectedItem = selection
            Me.EventBindingBox.Enabled = selection IsNot Nothing
        End Sub
#End Region

#Region "Event handlers"
        Private Sub OnBindingChanged(ByVal value As EventBinding)
            Me.EventBindingBox.Fill(value, Me._Provider)
            Me.EventBindingBox.Enabled = value IsNot Nothing
        End Sub

        Private Sub NewToolStripButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NewToolStripButton.Click
            Dim NewBinding As New EventBinding(Me.EventDataForCreation)
            Me._Bindings.Add(NewBinding)
            Me.NotifyComponentChanged(Me.EventDataForCreation.EventSource)
            Me.RefreshBindings(NewBinding)
        End Sub
        Private Sub DelToolStripButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DelToolStripButton.Click
            If Me._BindingsController.SelectedItem Is Nothing Then Exit Sub
            Me._Bindings.Remove(Me._BindingsController.SelectedItem)
            Me._BindingsController.SelectedItem.Delete()
            Me.NotifyComponentChanged(Me._BindingsController.SelectedItem.Source)
            Me.RefreshBindings(Nothing)
        End Sub
        Private Sub ClearToolStripButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ClearToolStripButton.Click
            Dim ModifiedSources = Me._Bindings.DeleteAll()
            Me._Bindings.Clear()
            Me.NotifyComponentsChanged(ModifiedSources)
            Me.RefreshBindings(Nothing)
        End Sub

        Private Sub EventBindingBox_DataChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EventBindingBox.DataChanged
            Me.RefreshBindings(Me._BindingsController.SelectedItem)
        End Sub
#End Region

    End Class

End Namespace
