﻿Imports System.ComponentModel.Design
Imports EventBindings.Extensions
Namespace Design
    Public Class HandlingDataBox

#Region "Fields"
        Private _HandlerProviders As ComboBoxController(Of IEventHandlerProvider)
        Private _HandlerSources As ComboBoxController(Of Object)
        Private _Handlers As ListBoxController(Of Object)

        Private _Data As HandlingData
        Private _Binding As EventBinding
        Private _ChangeService As IComponentChangeService
#End Region

#Region "Events"
        Public Event DataChanged As EventHandler
#End Region

#Region "Constructors"
        Public Sub New()

            ' Cet appel est requis par le concepteur.
            InitializeComponent()

            ' Ajoutez une initialisation quelconque après l'appel InitializeComponent().
            Me._HandlerProviders = New ComboBoxController(Of IEventHandlerProvider)(Me.cbbHandlingProvider, AddressOf OnHandlerProviderChanged)
            Me._HandlerSources = New ComboBoxController(Of Object)(Me.cbbHandlerSource, AddressOf OnHandlerSourceChanged)
            Me._Handlers = New ListBoxController(Of Object)(Me.lstHandlers, AddressOf OnHandlerChanged)
            Me.ErrorProvider1.Icon = SystemIcons.Warning
        End Sub
#End Region

#Region "Methods"
        Public Sub Fill(ByVal binding As EventBinding, ByVal provider As IEventBindingServiceProvider)
            If binding Is Nothing Then Throw New ArgumentNullException("binding")
            Me.Fill(binding.Handling, provider)
            Me._Binding = binding
        End Sub
        Public Sub Fill(ByVal handling As HandlingData, ByVal provider As IEventBindingServiceProvider)
            If handling Is Nothing Then Throw New ArgumentNullException("handling")
            Me._Data = handling
            Me._ChangeService = provider.GetService(Of IComponentChangeService)()
            Me.SetReferenceService(provider.GetService(Of IReferenceService))
            Dim Source = Me._Data.EventHandlerSource
            Dim ID = Me._Data.EventHandlerID
            Me._HandlerProviders.Items = provider.HandlerProviders
            Me._HandlerProviders.SelectedItem = Me._Data.Provider
            Me._HandlerSources.SelectedItem = Source
            Me._Handlers.SelectedItem = ID
        End Sub
        Private Sub SetReferenceService(ByVal value As IReferenceService)
            Me._HandlerProviders.ReferenceService = value
            Me._HandlerSources.ReferenceService = value
            Me._Handlers.ReferenceService = value
        End Sub
        Private Sub RaiseDataChanged()
            If Me._Binding IsNot Nothing AndAlso Me._Binding.Source IsNot Nothing AndAlso Me._ChangeService IsNot Nothing Then
                Me._ChangeService.OnComponentChanged(Me._Binding.Source, Nothing, Nothing, Nothing)
            End If
            RaiseEvent DataChanged(Me, EventArgs.Empty)
        End Sub
        Public Sub RefreshWarning()
            Dim Warning As String = ""
            If Me._Data IsNot Nothing AndAlso Me._Data.Provider IsNot Nothing Then
                Try
                    If Not Me._Data.Provider.CanResolveHandler(Me._Data.EventHandlerID, Me._Data.EventHandlerSource) Then
                        Warning = "Ce fournisseur ne peut résoudre cet handler sur cette source!"
                    End If
                Catch ex As Exception
                    Warning = ex.Message
                End Try
            End If
            Me.ErrorProvider1.SetError(Me.HandlerLabel, Warning)
        End Sub
#End Region

#Region "Event handlers"
        Private Sub OnHandlerProviderChanged(ByVal value As IEventHandlerProvider)
            Me._Data.Provider = value
            If value IsNot Nothing Then
                Me._HandlerSources.Items = value.GetEventHandlerSources.ToObjects
                If Me._HandlerSources.Items.IsNullOrEmpty Then
                    Me._HandlerSources.SelectedItem = Nothing
                Else
                    Me._HandlerSources.SelectedItem = Me._Handlers.Items.TryFirst
                End If
            Else
                Me._HandlerSources.Items = Nothing
                Me._HandlerSources.SelectedItem = Nothing
            End If
            Me.cbbHandlerSource.Enabled = Not Me._HandlerSources.Items.IsNullOrEmpty
            Me.OnHandlerSourceChanged(Me._HandlerSources.SelectedItem)
            Me.RaiseDataChanged()
            Me.RefreshWarning()
        End Sub
        Private Sub OnHandlerSourceChanged(ByVal value As Object)
            Me._Data.EventHandlerSource = value
            If Me._Data.Provider Is Nothing Then
                Me._Handlers.Items = Nothing
            Else
                Me._Handlers.Items = Me._Data.Provider.GetEventHandlerIDs(value).ToObjects
            End If
            If Me._Handlers.Items.IsNullOrEmpty Then
                Me._Handlers.SelectedItem = Nothing
            Else
                Me._Handlers.SelectedItem = Me._Handlers.Items.TryFirst
            End If
            Me.RaiseDataChanged()
            Me.RefreshWarning()
        End Sub
        Private Sub OnHandlerChanged(ByVal value As Object)
            Me._Data.EventHandlerID = value
            Me.RaiseDataChanged()
            Me.RefreshWarning()
        End Sub
#End Region

    End Class
End Namespace

