﻿Imports System.ComponentModel.Design
Imports EventBindings.Extensions

Namespace Design
    Public MustInherit Class ListControlController(Of TItem, TControl As ListControl)

#Region "Fields"
        Protected _Items As IEnumerable(Of TItem)
        Protected _SelectedItem As TItem
        Protected _ListControl As TControl
#End Region

#Region "Constructors"
        Public Sub New(ByVal listControl As TControl)
            If listControl Is Nothing Then Throw New ArgumentNullException("listControl")
            Me._ListControl = listControl
            AddHandler Me._ListControl.Format, AddressOf OnFormat
        End Sub
        Public Sub New(ByVal listControl As TControl, ByVal selectionHandler As Action(Of TItem))
            Me.New(listControl)
            Me.SelectionHandler = selectionHandler
        End Sub
#End Region

#Region "Properties"
        Public Property SelectionHandler As Action(Of TItem)
        Public Property ReferenceService As IReferenceService
        Public Property Items As IEnumerable(Of TItem)
            Get
                Return Me._Items
            End Get
            Set(ByVal value As IEnumerable(Of TItem))
                Me.RemoveSelectionChangedEventHandler()
                Me._Items = value
                Me.FillControl()
                Me.AddSelectionChangedEventHandler()
            End Set
        End Property
        Public Property SelectedItem As TItem
            Get
                Return Me._SelectedItem
            End Get
            Set(ByVal value As TItem)
                Me._SelectedItem = value
                Me.SetControlSelection()
            End Set
        End Property
#End Region

#Region "Methods"
        Protected MustOverride Sub FillControl()
        Protected MustOverride Sub SetControlSelection()
        Protected MustOverride Function GetSelectedItem() As TItem

        Protected Sub AddSelectionChangedEventHandler()
            AddHandler Me._ListControl.SelectedValueChanged, AddressOf OnSelectedValueChanged
        End Sub
        Protected Sub RemoveSelectionChangedEventHandler()
            RemoveHandler Me._ListControl.SelectedValueChanged, AddressOf OnSelectedValueChanged
        End Sub

        Protected Sub OnFormat(ByVal sender As Object, ByVal e As ListControlConvertEventArgs)
            e.Value = Me.ReferenceService.GetAnyName(e.ListItem)
        End Sub
        Protected Sub OnSelectedValueChanged(ByVal sender As Object, ByVal e As System.EventArgs)
            Me._SelectedItem = Me.GetSelectedItem
            If Me.SelectionHandler Is Nothing Then Exit Sub
            Me.SelectionHandler.Invoke(Me._SelectedItem)
        End Sub
#End Region

    End Class
End Namespace
