﻿Namespace Extensions
    <Extension()> _
    Public Module CollectionExtensions

#Region "IEnumerable"
        <Extension()>
        Public Function ToObjects(ByVal E As IEnumerable) As Object()
            Dim Lst As New List(Of Object)
            If E IsNot Nothing Then
                For Each Item In E
                    Lst.Add(Item)
                Next
            End If
            Return Lst.ToArray
        End Function

        <Extension()> _
        Public Function IsNullOrEmpty(ByVal E As IEnumerable) As Boolean
            Return E Is Nothing OrElse Not E.GetEnumerator.MoveNext
        End Function
#End Region

#Region "IEnumerable(Of )"
        <Extension()> _
        Public Function IsNullOrEmpty(Of T)(ByVal E As IEnumerable(Of T)) As Boolean
            Return E Is Nothing OrElse Not E.Any
        End Function
        <Extension()> _
        Public Function Contains(Of T)(ByVal LST As IEnumerable(Of T), ByVal condition As Func(Of T, Boolean)) As Boolean
            If condition Is Nothing Then Throw New ArgumentNullException("condition")
            If LST Is Nothing Then Return False
            For Each Item In LST
                If condition(Item) Then Return True
            Next
            Return False
        End Function
        <Extension()> _
        Public Function Take(Of T)(ByVal LST As IEnumerable(Of T), ByVal condition As Func(Of T, Boolean)) As T
            If condition Is Nothing Then Throw New ArgumentNullException("condition")
            If LST Is Nothing Then Return Nothing
            For Each Item In LST
                If condition(Item) Then Return Item
            Next
            Return Nothing
        End Function
        <Extension()> _
        Public Function TryFirst(Of T)(ByVal LST As IEnumerable(Of T)) As T
            If LST.IsNullOrEmpty Then Return Nothing
            Return LST.First
        End Function
#End Region

#Region "IList"
        <Extension()> _
        Public Sub AddDistinct(ByVal LST As IList, ByVal item As Object)
            If LST Is Nothing Then Throw New ArgumentNullException("LST")
            If LST.Contains(item) Then Exit Sub
            LST.Add(item)
        End Sub
        <Extension()> _
        Public Sub AddDistinctRange(ByVal LST As IList, ByVal range As IEnumerable)
            If LST Is Nothing Then Throw New ArgumentNullException("LST")
            If range.IsNullOrEmpty Then Exit Sub
            For Each Item In range
                If LST.Contains(Item) Then Continue For
                LST.Add(Item)
            Next
        End Sub
       
#End Region

#Region "IList(Of )"
        <Extension()> _
        Public Sub AddDistinct(Of TItem)(ByVal LST As IList(Of TItem), ByVal item As TItem)
            If LST Is Nothing Then Throw New ArgumentNullException("LST")
            If LST.Contains(item) Then Exit Sub
            LST.Add(item)
        End Sub
        <Extension()> _
        Public Sub AddDistinctRange(Of TItem)(ByVal LST As IList(Of TItem), ByVal range As IEnumerable(Of TItem))
            If LST Is Nothing Then Throw New ArgumentNullException("LST")
            If range.IsNullOrEmpty Then Exit Sub
            For Each Item In range
                If LST.Contains(Item) Then Continue For
                LST.Add(Item)
            Next
        End Sub
#End Region

#Region "IDictionary(Of )"
        <Extension()>
        Public Function TryGetItem(Of TKey, TItem)(ByVal DIC As IDictionary(Of TKey, TItem),
            ByVal key As TKey, Optional ByVal defaultItem As TItem = Nothing) As TItem
            If DIC Is Nothing Then Return defaultItem
            If key Is Nothing Then Return defaultItem
            If Not DIC.ContainsKey(key) Then Return defaultItem
            Return DIC(key)
        End Function
#End Region

    End Module
End Namespace

