﻿Namespace Extensions
    <Extension()>
    Public Module EventBindingExtensions

#Region "EventBinding & IEnumerable(of )"
        <Extension()>
        Public Sub Delete(ByVal BINDING As EventBinding)
            If BINDING Is Nothing Then Exit Sub
            BINDING.Source = Nothing
        End Sub
        <Extension()>
        Public Function DeleteAll(ByVal BINDINGS As IEnumerable(Of EventBinding)) As IEnumerable(Of IEventBindingSource)
            If BINDINGS.IsNullOrEmpty Then Return Nothing
            Dim ModifiedSources As New List(Of IEventBindingSource)
            For Each Binding In BINDINGS
                If Binding.Source IsNot Nothing AndAlso Not ModifiedSources.Contains(Binding.Source) Then
                    ModifiedSources.Add(Binding.Source)
                End If
                Binding.Delete()
            Next
            Return ModifiedSources
        End Function
#End Region

#Region "IEventMediatorFactory"
        <Extension()>
        Function SupportMediationFor(ByVal F As IEventMediatorFactory, ByVal binding As EventBinding) As Boolean
            If F Is Nothing Then Return False
            If binding Is Nothing Then Return False
            Return F.SupportMediationFor(binding.Event, binding.Handling, binding.Mediation.MediationMode)
        End Function
        <Extension()>
        Function CreateMediator(ByVal FACTORY As IEventMediatorFactory, ByVal binding As EventBinding) As IEventMediator
            If FACTORY Is Nothing Then Throw New ArgumentNullException("FACTORY")
            If binding Is Nothing Then Throw New ArgumentNullException("binding")
            Return FACTORY.CreateMediator(binding.Event, binding.Handling, binding.Mediation.MediationMode)
        End Function
#End Region

#Region "IEventBindingServiceProvider"
        <Extension()>
        Public Function GetBindings(ByVal SP As IEventBindingServiceProvider, ByVal [event] As EventData) As EventBinding()
            Dim Bindings As New List(Of EventBinding)
            For Each Source In SP.GetBindingSources
                Dim SourceBindings = Source.GetBindings([event])
                If SourceBindings.IsNullOrEmpty Then Continue For
                Bindings.AddRange(SourceBindings)
            Next
            Return Bindings.ToArray
        End Function
        <Extension()>
        Public Sub ResetBindings(ByVal SP As IEventBindingServiceProvider, ByVal [event] As EventData)
            For Each Source In SP.GetBindingSources
                Source.ResetBindings([event])
            Next
        End Sub
#End Region

#Region "IEventBindingSource"
        <Extension()>
        Public Function GetBindableBindings(ByVal SOURCE As IEventBindingSource) As IEnumerable(Of EventBinding)
            If SOURCE Is Nothing Then Return Nothing
            Dim Query = From B In SOURCE.GetBindings Where B.CanBind
            Return Query.ToList
        End Function
#End Region

#Region "IEnumerable(Of ILifeObserver(Of TSubject))"
        <Extension()>
        Public Sub NotifyCreating(Of TSubject)(ByVal LST As IEnumerable(Of ILifeObserver(Of TSubject)), ByVal owner As EventBinding)
            If LST Is Nothing Then Exit Sub
            For Each Observer In LST
                Observer.OnCreating(owner)
            Next
        End Sub
        <Extension()>
        Public Sub NotifyCreated(Of TSubject)(ByVal LST As IEnumerable(Of ILifeObserver(Of TSubject)), ByVal owner As EventBinding, ByVal subject As TSubject)
            If LST Is Nothing Then Exit Sub
            For Each Observer In LST
                Observer.OnCreated(owner, subject)
            Next
        End Sub
        <Extension()>
        Public Sub NotifyDisposing(Of TSubject)(ByVal LST As IEnumerable(Of ILifeObserver(Of TSubject)), ByVal owner As EventBinding, ByVal subject As TSubject)
            If LST Is Nothing Then Exit Sub
            For Each Observer In LST
                Observer.OnDisposing(owner, subject)
            Next
        End Sub
        <Extension()>
        Public Sub NotifyDisposed(Of TSubject)(ByVal LST As IEnumerable(Of ILifeObserver(Of TSubject)), ByVal owner As EventBinding)
            If LST Is Nothing Then Exit Sub
            For Each Observer In LST
                Observer.OnDisposed(owner)
            Next
        End Sub
#End Region
    End Module
End Namespace

