﻿Imports System.ComponentModel.Design
Namespace Extensions
    <Extension()> _
    Public Module ServiceExtensions

#Region "IServiceProvider"
        <Extension()>
        Public Function GetService(Of TService As Class)(ByVal SP As IServiceProvider) As TService
            If SP Is Nothing Then Return Nothing
            Return TryCast(SP.GetService(GetType(TService)), TService)
        End Function
        <Extension()>
        Public Function GetOrCreateEventBindingServiceProvider(ByVal SP As IServiceProvider) As IEventBindingServiceProvider
            Dim EventBindingSP = SP.GetService(Of IEventBindingServiceProvider)()
            If EventBindingSP IsNot Nothing Then Return EventBindingSP
            Dim DefaultSP = New EventBindingServiceProvider(SP)
            DefaultSP.Promote()
            Return DefaultSP
        End Function

        <Extension()>
        Public Function TryGetService(Of TService)(ByVal SP As IServiceProvider, ByRef service As TService) As Boolean
            Dim Candidat = SP.GetService(GetType(TService))
            If Candidat Is Nothing Then Return False
            service = DirectCast(Candidat, TService)
            Return True
        End Function
#End Region

#Region "IEventBindingServiceProvider"
        <Extension()>
        Public Sub Promote(ByVal SP As IEventBindingServiceProvider)
            Dim Container = SP.GetService(Of IServiceContainer)()
            Container.RemoveService(GetType(IEventBindingServiceProvider), True)
            Container.AddService(GetType(IEventBindingServiceProvider), SP, True)
        End Sub
#End Region

#Region "IDesignerHost"
        <Extension()>
        Public Function GetComponents(Of TComponent)(ByVal HOST As IDesignerHost) As IEnumerable(Of TComponent)
            Return GetComponents(Of TComponent)(HOST, Function(C As TComponent) (True))
        End Function
        <Extension()>
        Public Function GetComponents(ByVal HOST As IDesignerHost, ByVal includeCondition As Func(Of IComponent, Boolean)) As IEnumerable(Of IComponent)
            Return GetComponents(Of IComponent)(HOST, includeCondition)
        End Function
        <Extension()>
        Public Function GetComponents(Of TComponent)(ByVal HOST As IDesignerHost, ByVal includeCondition As Func(Of TComponent, Boolean)) As IEnumerable(Of TComponent)
            If HOST Is Nothing OrElse HOST.Container Is Nothing Then Return Nothing
            Dim Components As New List(Of TComponent)
            For Each Candidat As IComponent In HOST.Container.Components
                If Not TypeOf Candidat Is TComponent Then Continue For
                Dim Casted = DirectCast(Candidat, TComponent)
                If Not includeCondition(Casted) Then Continue For
                Components.Add(Casted)
            Next
            If Components.Count = 0 Then Return Nothing
            Return Components
        End Function
#End Region

#Region "IReferenceService"
        <Extension()>
        Public Function GetAnyName(ByVal RS As IReferenceService, ByVal component As Object) As String
            If component Is Nothing Then Return String.Empty
            If RS Is Nothing Then Return component.ToString
            If TypeOf component Is EventBinding Then
                Return DirectCast(component, EventBinding).GetName(RS)
            End If
            Dim Name = RS.GetName(component)
            If String.IsNullOrWhiteSpace(Name) Then Return component.ToString
            Return Name
        End Function
#End Region

    End Module
End Namespace

