﻿Imports EventBindings.Extensions
Public Structure EventData

#Region "Fields"
    Private _EventName As String
    Private _EventSource As Object
    Private _EventDescriptor As EventDescriptor
#End Region

#Region "Constructors"
    Public Sub New(ByVal eventName As String, ByVal eventSource As Object)
        If String.IsNullOrWhiteSpace(eventName) Then Throw New ArgumentNullException("eventName")
        If eventSource Is Nothing Then Throw New ArgumentNullException("eventSource")
        Me._EventName = eventName
        Me._EventSource = eventSource
    End Sub
    Public Sub New(ByVal eventDescriptor As EventDescriptor, ByVal eventSource As Object)
        If eventDescriptor Is Nothing Then Throw New ArgumentNullException("eventDescriptor")
        If eventSource Is Nothing Then Throw New ArgumentNullException("eventSource")
        Me._EventDescriptor = eventDescriptor
        Me._EventName = eventDescriptor.Name
        Me._EventSource = eventSource
    End Sub
#End Region

#Region "Properties"
    Public ReadOnly Property EventName As String
        Get
            Return Me._EventName
        End Get
    End Property
    Public ReadOnly Property EventSource As Object
        Get
            Return Me._EventSource
        End Get
    End Property
    Public ReadOnly Property EventDescriptor As EventDescriptor
        Get
            If Me._EventDescriptor Is Nothing Then
                Me._EventDescriptor = TypeDescriptor.GetEvents(EventSource).GetEvent(Me._EventName, StringComparison.InvariantCultureIgnoreCase)
            End If
            Return Me._EventDescriptor
        End Get
    End Property
#End Region

#Region "Methods"
    Public Function IsValidEvent() As Boolean
        Return Me.EventDescriptor IsNot Nothing
    End Function
    Public Overrides Function GetHashCode() As Integer
        Return Me._EventSource.GetHashCode Xor Me._EventName.GetHashCode
    End Function
    Public Sub AddEventHandler(ByVal handler As [Delegate])
        Me._EventDescriptor.AddEventHandler(Me._EventSource, handler)
    End Sub
    Public Sub RemoveEventHandler(ByVal handler As [Delegate])
        Me._EventDescriptor.RemoveEventHandler(Me._EventSource, handler)
    End Sub
#End Region

End Structure
