﻿Imports System.ComponentModel.Design
Imports EventBindings.Extensions

Public Class HandlingData
    Implements ILifeObservableContainer

#Region "Fields"
    Private _EventHandlerID As Object
    Private _EventHandlerSource As Object
    Private _Provider As IEventHandlerProvider
    Private _EventHandler As Object
    Private _EventHandlerType As Type
#End Region

#Region "Properties"
    <Browsable(False)>
    Public ReadOnly Property [ReadOnly] As Boolean
        Get
            Return Me._EventHandler IsNot Nothing
        End Get
    End Property
    <Editor(GetType(Design.EventHandlerIDTypeEditor), GetType(System.Drawing.Design.UITypeEditor))>
    Public Property EventHandlerID As Object
        Get
            Return Me._EventHandlerID
        End Get
        Set(ByVal value As Object)
            If Me.ReadOnly Then Throw New ReadOnlyException
            Me._EventHandlerID = value
            Me.ClearHandler()
        End Set
    End Property
    <Editor(GetType(Design.EventBindingSourceTypeEditor), GetType(System.Drawing.Design.UITypeEditor))>
    Public Property EventHandlerSource As Object
        Get
            Return Me._EventHandlerSource
        End Get
        Set(ByVal value As Object)
            If Me.ReadOnly Then Throw New ReadOnlyException
            Me._EventHandlerSource = value
            Me.ClearHandler()
            Me.EventHandlerID = Nothing
        End Set
    End Property
    <Editor(GetType(Design.EventHandlerProviderTypeEditor), GetType(System.Drawing.Design.UITypeEditor))>
    Public Property Provider As IEventHandlerProvider
        Get
            Return Me._Provider
        End Get
        Set(ByVal value As IEventHandlerProvider)
            If Me.ReadOnly Then Throw New ReadOnlyException
            Me._Provider = value
            Me.ClearHandler()
            Me.EventHandlerID = Nothing
            Me.EventHandlerSource = Nothing
        End Set
    End Property
    <Browsable(False)>
    Public ReadOnly Property EventHandler As Object
        Get
            Return Me._EventHandler
        End Get
    End Property
    Public ReadOnly Property EventHandlerType As Type
        Get
            If Me._EventHandlerType Is Nothing AndAlso Me.CanCreateHandler Then
                Me._EventHandlerType = Me._Provider.ResolveHandlerType(Me._EventHandlerID, Me._EventHandlerSource)
            End If
            Return Me._EventHandlerType
        End Get
    End Property
    <Browsable(False)>
    Public ReadOnly Property NeedHandlerCreation As Boolean
        Get
            Return Me._EventHandler Is Nothing
        End Get
    End Property
#End Region

#Region "Methods"
    Public Sub ClearHandler()
        Me._EventHandler = Nothing
    End Sub
    Public Function CanCreateHandler() As Boolean
        If Me._Provider Is Nothing Then Return False
        Return Me._Provider.CanResolveHandler(Me._EventHandlerID, Me._EventHandlerSource)
    End Function
    Public Sub CreateHandler()
        Me._EventHandler = Me._Provider.ResolveHandler(Me._EventHandlerID, Me._EventHandlerSource)
    End Sub
    Private Function GetOwnerName(ByVal referenceService As IReferenceService) As String
        Dim ProviderName = referenceService.GetAnyName(Me._Provider)
        Dim SourceName = referenceService.GetAnyName(Me._EventHandlerSource)
        If String.IsNullOrWhiteSpace(SourceName) Then
            Return ProviderName
        Else
            Return String.Format("{0}.{1}", ProviderName, SourceName)
        End If
    End Function
    Public Function GetName(ByVal referenceService As IReferenceService) As String
        Dim IDName = referenceService.GetAnyName(Me._EventHandlerID)
        Dim OwnerName = Me.GetOwnerName(referenceService)
        If String.IsNullOrWhiteSpace(IDName) Then
            If String.IsNullOrWhiteSpace(OwnerName) Then Return String.Empty
            Return "On " + OwnerName
        End If
        If String.IsNullOrWhiteSpace(OwnerName) Then Return IDName
        Return String.Format("{0} On {1}", IDName, OwnerName)
    End Function
    Public Function GetLifeObservers(Of TSubject)() As System.Collections.Generic.IEnumerable(Of ILifeObserver(Of TSubject)) Implements ILifeObservableContainer.GetLifeObservers
        Dim Candidates As New List(Of ILifeObserver(Of TSubject))
        If TypeOf Me._Provider Is ILifeObserver(Of TSubject) Then
            Candidates.Add(DirectCast(Me._Provider, ILifeObserver(Of TSubject)))
        End If
        If TypeOf Me._EventHandlerSource Is ILifeObserver(Of TSubject) Then
            Candidates.Add(DirectCast(Me._EventHandlerSource, ILifeObserver(Of TSubject)))
        End If
        If TypeOf Me._EventHandlerID Is ILifeObserver(Of TSubject) Then
            Candidates.Add(DirectCast(Me._EventHandlerID, ILifeObserver(Of TSubject)))
        End If
        If TypeOf Me._EventHandler Is ILifeObserver(Of TSubject) Then
            Candidates.Add(DirectCast(Me._EventHandler, ILifeObserver(Of TSubject)))
        End If
        Return Candidates
    End Function
#End Region

End Class
