﻿Public Class MediationData
    Implements ILifeObservableContainer

#Region "Fields"
    Private _Factory As IEventMediatorFactory
    Private _Mediator As IEventMediator
    Private _MediationMode As MediationMode
#End Region

#Region "Constructor"
    Public Sub New()
        Me._MediationMode = MediationMode.Synchronous
    End Sub
    Public Sub New(ByVal factory As IEventMediatorFactory, Optional ByVal mediationMode As MediationMode = MediationMode.Synchronous)
        Me._Factory = factory
        Me._MediationMode = mediationMode
    End Sub
#End Region

#Region "Properties"
    <Browsable(False)>
    Public ReadOnly Property [ReadOnly] As Boolean
        Get
            Return Me._Mediator IsNot Nothing
        End Get
    End Property
    Public Property MediationMode As MediationMode
        Get
            Return Me._MediationMode
        End Get
        Set(ByVal value As MediationMode)
            If Me.ReadOnly Then Throw New ReadOnlyException
            Me._MediationMode = value
        End Set
    End Property
    <Editor(GetType(Design.MediatorFactoryTypeEditor), GetType(System.Drawing.Design.UITypeEditor))>
    Public Property Factory As IEventMediatorFactory
        Get
            Return Me._Factory
        End Get
        Set(ByVal value As IEventMediatorFactory)
            If Me.ReadOnly Then Throw New ReadOnlyException
            Me._Factory = value
            Me.ClearMediator()
        End Set
    End Property
    <Browsable(False)>
    Public ReadOnly Property NeedMediatorCreation As Boolean
        Get
            Return Me._Mediator Is Nothing
        End Get
    End Property
    <Browsable(False)>
    Public ReadOnly Property Mediator As IEventMediator
        Get
            Return Me._Mediator
        End Get
    End Property
#End Region

#Region "Methods"
    Public Sub ClearMediator()
        Me._Mediator = Nothing
    End Sub
    Public Function CanCreateMediator(ByVal [event] As EventData, ByVal handler As HandlingData) As Boolean
        If Me._Factory Is Nothing Then Return False
        Return Me._Factory.SupportMediationFor([event], handler, Me._MediationMode)
    End Function
    Public Sub CreateMediator(ByVal [event] As EventData, ByVal handler As HandlingData)
        Me._Mediator = Me._Factory.CreateMediator([event], handler, Me._MediationMode)
    End Sub
    Public Function GetLifeObservers(Of TSubject)() As System.Collections.Generic.IEnumerable(Of ILifeObserver(Of TSubject)) Implements ILifeObservableContainer.GetLifeObservers
        Dim Candidates As New List(Of ILifeObserver(Of TSubject))
        If TypeOf Me._Factory Is ILifeObserver(Of TSubject) Then
            Candidates.Add(DirectCast(Me._Factory, ILifeObserver(Of TSubject)))
        End If
        If TypeOf Me._Mediator Is ILifeObserver(Of TSubject) Then
            Candidates.Add(DirectCast(Me._Mediator, ILifeObserver(Of TSubject)))
        End If
        Return Candidates
    End Function
#End Region

    
End Class
