﻿Imports EventBindings.Extensions

Public Class EventBindingDiscoveryService
    Implements IEventBindingDiscoveryService

    Private _ServiceProvider As IEventBindingServiceProvider

    Public Sub New(ByVal serviceProvider As IEventBindingServiceProvider)
        If serviceProvider Is Nothing Then Throw New ArgumentNullException("serviceProvider")
        Me._ServiceProvider = serviceProvider
    End Sub

    Public Function GetDefaultEventProperty(ByVal component As Object) As PropertyDescriptor Implements IEventBindingDiscoveryService.GetDefaultEventProperty
        If component Is Nothing Then Throw New ArgumentNullException("component")

        Dim DefaultEvent = TypeDescriptor.GetDefaultEvent(component)
        If DefaultEvent Is Nothing Then Return Nothing
        Return New EventBindingDescriptor(New EventData(DefaultEvent, component), Me._ServiceProvider)
    End Function

    Public Function GetEventProperties(ByVal component As Object, Optional ByVal attributes() As System.Attribute = Nothing) As PropertyDescriptorCollection Implements IEventBindingDiscoveryService.GetEventProperties
        If component Is Nothing Then Throw New ArgumentNullException("component")

        If attributes.IsNullOrEmpty Then
            Return GetEventProperties(component, TypeDescriptor.GetEvents(component))
        Else
            Return GetEventProperties(component, TypeDescriptor.GetEvents(component, attributes))
        End If
    End Function
    Private Function GetEventProperties(ByVal component As Object, ByVal events As EventDescriptorCollection) As PropertyDescriptorCollection
        If component Is Nothing Then Throw New ArgumentNullException("component")
        If events.IsNullOrEmpty Then Return Nothing

        Dim Descriptors As New List(Of PropertyDescriptor)
        For Each Descriptor As EventDescriptor In events
            Descriptors.Add(New EventBindingDescriptor(New EventData(Descriptor, component), Me._ServiceProvider))
        Next
        Return New PropertyDescriptorCollection(Descriptors.ToArray)
    End Function

    Public Function HasEvents(ByVal component As Object) As Boolean Implements IEventBindingDiscoveryService.HasEvents
        If component Is Nothing Then Return False
        Return Not TypeDescriptor.GetEvents(component).IsNullOrEmpty
    End Function
End Class
