﻿Imports System.ComponentModel.Design
Imports EventBindings.Extensions

Public Class EventBindingServiceProvider
    Implements IEventBindingServiceProvider

#Region "Fields"
    Private _InnerProvider As IServiceProvider

    Private _BindingSources As List(Of IEventBindingSource)
    Private _HandlerProviders As List(Of IEventHandlerProvider)
    Private _MediatorFactories As List(Of IEventMediatorFactory)

    Private _DiscoveryService As IEventBindingDiscoveryService
#End Region

#Region "Constructors"
    Public Sub New(ByVal provider As IServiceProvider)
        If provider Is Nothing Then Throw New ArgumentNullException("provider")

        Me._BindingSources = New List(Of IEventBindingSource)
        Me._HandlerProviders = New List(Of IEventHandlerProvider)
        Me._MediatorFactories = New List(Of IEventMediatorFactory)
        Me._InnerProvider = provider
    End Sub
#End Region

#Region "Properties"
    Public ReadOnly Property DiscoryService As IEventBindingDiscoveryService Implements IEventBindingServiceProvider.DiscoryService
        Get
            If Me._DiscoveryService Is Nothing Then
                Me._DiscoveryService = Me._InnerProvider.GetService(Of IEventBindingDiscoveryService)()
            End If
            If Me._DiscoveryService Is Nothing Then
                Me._DiscoveryService = New EventBindingDiscoveryService(Me)
            End If
            Return Me._DiscoveryService
        End Get
    End Property
    Public ReadOnly Property BindingSources As System.Collections.Generic.IList(Of IEventBindingSource) Implements IEventBindingServiceProvider.BindingSources
        Get
            Return Me._BindingSources
        End Get
    End Property
    Public ReadOnly Property HandlerProviders As System.Collections.Generic.IList(Of IEventHandlerProvider) Implements IEventBindingServiceProvider.HandlerProviders
        Get
            Return Me._HandlerProviders
        End Get
    End Property
    Public ReadOnly Property MediatorFactories As System.Collections.Generic.IList(Of IEventMediatorFactory) Implements IEventBindingServiceProvider.MediatorFactories
        Get
            Return Me._MediatorFactories
        End Get
    End Property
#End Region

#Region "Methods"
    Public Function GetService(ByVal serviceType As Type) As Object Implements IServiceProvider.GetService
        If serviceType Is GetType(IEventBindingServiceProvider) Then Return Me
        If serviceType Is GetType(IEventBindingDiscoveryService) AndAlso Me._DiscoveryService IsNot Nothing Then Return Me._DiscoveryService
        Return Me._InnerProvider.GetService(serviceType)
    End Function

    Public Function GetBindingSources() As IEventBindingSource() Implements IEventBindingServiceProvider.GetBindingSources
        Return Me._BindingSources.ToArray
    End Function
    Public Function GetEventHandlerProviders() As IEventHandlerProvider() Implements IEventBindingServiceProvider.GetEventHandlerProviders
        Return Me._HandlerProviders.ToArray
    End Function
    Public Function GetMediatorFactories() As IEventMediatorFactory() Implements IEventBindingServiceProvider.GetMediatorFactories
        Return Me._MediatorFactories.ToArray
    End Function

    Public Sub RetrieveServicesFromHostView() Implements IEventBindingServiceProvider.RetrieveServicesFromHostView
        Dim Host = Me._InnerProvider.GetService(Of IDesignerHost)()
        Me._BindingSources.AddDistinctRange(Host.GetComponents(Of IEventBindingSource))
        Me._HandlerProviders.AddDistinctRange(Host.GetComponents(Of IEventHandlerProvider))
        Me._MediatorFactories.AddDistinctRange(Host.GetComponents(Of IEventMediatorFactory))
    End Sub
#End Region

End Class
